#include <iostream>
#include <fstream>
#include <iomanip>
#include <cstring>
#include <cstdint>
#include <cstddef>
#include <string>

// 1. &#1060;&#1091;&#1085;&#1082;&#1094;&#1080;&#1080; &#1076;&#1083;&#1103; &#1087;&#1077;&#1095;&#1072;&#1090;&#1080; &#1076;&#1072;&#1085;&#1085;&#1099;&#1093;
void print_in_hex(uint8_t byte) {
    std::cout << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(byte) << ' ';
}

void print_in_hex(const void* data, size_t size) {
    const uint8_t* bytes = static_cast<const uint8_t*>(data);
    for (size_t i = 0; i < size; ++i) {
        print_in_hex(bytes[i]);
        if ((i + 1) % 16 == 0) std::cout << '\n';
    }
    std::cout << '\n';
}

void print_in_binary(uint8_t byte) {
    for (int i = 7; i >= 0; --i) {
        std::cout << ((byte >> i) & 1);
    }
    std::cout << ' ';
}

void print_in_binary(const void* data, size_t size) {
    const uint8_t* bytes = static_cast<const uint8_t*>(data);
    for (size_t i = 0; i < size; ++i) {
        print_in_binary(bytes[i]);
        if ((i + 1) % 4 == 0) std::cout << '\n';
    }
    std::cout << '\n';
}

// 2. &#1050;&#1072;&#1083;&#1100;&#1082;&#1091;&#1083;&#1103;&#1090;&#1086;&#1088; &#1076;&#1083;&#1103; &#1087;&#1086;&#1073;&#1080;&#1090;&#1086;&#1074;&#1099;&#1093; &#1086;&#1087;&#1077;&#1088;&#1072;&#1094;&#1080;&#1081;
void bitwise_calculator() {
    uint16_t operand1, operand2;
    char operation;

    std::cout << "Enter the first operand: ";
    std::cin >> operand1;
    std::cout << "Enter the operator (&, |, ^): ";
    std::cin >> operation;
    std::cout << "Enter the second operand: ";
    std::cin >> operand2;

    uint16_t result;
    switch (operation) {
    case '&': result = operand1 & operand2; break;
    case '|': result = operand1 | operand2; break;
    case '^': result = operand1 ^ operand2; break;
    default:
        std::cerr << "Invalid operator!\n";
        return;
    }

    std::cout << "Result in hexadecimal format: ";
    print_in_hex(&result, sizeof(result));

    std::cout << "Result in binary format: ";
    print_in_binary(&result, sizeof(result));
}

// 3. &#1056;&#1072;&#1073;&#1086;&#1090;&#1072; &#1089;&#1086; &#1089;&#1090;&#1088;&#1091;&#1082;&#1090;&#1091;&#1088;&#1072;&#1084;&#1080;
struct Student {
    char name[17];
    uint16_t year_of_admission;
    float gpa;
    unsigned gender : 1;
    uint8_t completed_courses;
    Student* group_leader;
};

void analyze_students() {
    Student students[3] = {
        {"Alice", 2020, 3.8, 0, 5, nullptr},
        {"Bob", 2020, 3.5, 1, 5, nullptr},
        {"Charlie", 2019, 4.0, 1, 6, nullptr}
    };

    students[0].group_leader = &students[2];
    students[1].group_leader = &students[2];

    std::cout << "Array address: " << &students << "\n";
    std::cout << "Array size: " << sizeof(students) << " bytes\n\n";

    for (size_t i = 0; i < 3; ++i) {
        std::cout << "Student " << i + 1 << ":\n";
        std::cout << "  Address: " << &students[i] << "\n";
        std::cout << "  Size: " << sizeof(students[i]) << " bytes\n";
    }

    Student& student = students[0];
    std::cout << "\nAnalysis of Alice's fields:\n";
    std::cout << "  Name: Address=" << static_cast<void*>(student.name)
        << ", Offset=" << offsetof(Student, name)
        << ", Size=" << sizeof(student.name) << '\n';
    std::cout << "  Year of admission: Address=" << &student.year_of_admission
        << ", Offset=" << offsetof(Student, year_of_admission)
        << ", Size=" << sizeof(student.year_of_admission) << '\n';
    std::cout << "  GPA: Address=" << &student.gpa
        << ", Offset=" << offsetof(Student, gpa)
        << ", Size=" << sizeof(student.gpa) << '\n';
    std::cout << "  Number of courses: Address=" << static_cast<void*>(&student.completed_courses)
        << ", Offset=" << offsetof(Student, completed_courses)
        << ", Size=" << sizeof(student.completed_courses) << '\n';
}

// 4. &#1054;&#1073;&#1088;&#1072;&#1073;&#1086;&#1090;&#1082;&#1072; &#1090;&#1077;&#1082;&#1089;&#1090;&#1086;&#1074;&#1086;&#1075;&#1086; &#1092;&#1072;&#1081;&#1083;&#1072;
void process_text_file() {
    const char* filename = "text.txt";

    std::ifstream file(filename, std::ios::binary);
    if (!file) {
        std::cerr << "File open error!\n";
        return;
    }

    file.seekg(0, std::ios::end);
    size_t filesize = file.tellg();
    file.seekg(0, std::ios::beg);

    char* buffer = new char[filesize];
    file.read(buffer, filesize);

    std::cout << "Enter the string to search for: ";
    char search_str[256];
    std::cin.ignore();
    std::cin.getline(search_str, 256);

    size_t count = 0;
    char* pos = buffer;
    while ((pos = strstr(pos, search_str)) != nullptr) {
        ++count;
        pos += strlen(search_str);
    }

    std::cout << "Number of occurrences: " << count << "\n";

    delete[] buffer;
}

int main() {
    std::cout << "Choose a task: 1 - Print, 2 - Calculator, 3 - Students, 4 - File\n";
    int choice;
    std::cin >> choice;

    switch (choice) {
    case 1:
        // &#1044;&#1077;&#1084;&#1086;&#1085;&#1089;&#1090;&#1088;&#1072;&#1094;&#1080;&#1103; &#1092;&#1091;&#1085;&#1082;&#1094;&#1080;&#1081; &#1087;&#1077;&#1095;&#1072;&#1090;&#1080;
    {
        uint16_t data = 0xABCD;
        std::cout << "Hexadecimal output:\n";
        print_in_hex(&data, sizeof(data));
        std::cout << "Binary output:\n";
        print_in_binary(&data, sizeof(data));
    }
    break;
    case 2:
        bitwise_calculator();
        break;
    case 3:
        analyze_students();
        break;
    case 4:
        process_text_file();
        break;
    default:
        std::cerr << "Invalid choice!\n";
    }

    return 0;
}
